#! /usr/bin/python

#created by James Martin (oss on the kjams forums/wiki), ceedvd at gmail dot com
#Released under the GNU Public License, Version 3.
#Use at your own risk, this software is not guaranteed.

import appscript
import time
import sys
import ConfigParser
import os

class kjams:
  
  def __init__(self):
    self.appscript = appscript.app('kJams Pro')
    self.name = 'kjams'
 
  def playpause(self):
    self.appscript.docommand(1)
  def play(self):
    self.appscript.docommand(1)
  def pause(self):
    self.appscript.docommand(1)             
  def stop(self):
    self.appscript.docommand(2)
    
  def next(self):
    self.appscript.docommand(4)
        
  def get_state(self):
    mode = self.appscript.get_mode()
    if mode == 0:
      self.state = 'stopped'
    elif mode == 1:
      self.state = 'playing'
    elif mode == 2:
      self.state = 'paused'
    else:
      print "This is an unknown state."
      sys.exit(1)
    return self.state
 
  def get_time(self):
    return self.appscript.get_time(2)
    
  def get_volume(self):
    return self.appscript.get_volume(0)  


  def set_volume(self, volume):
    self.appscript.set_volume(0, level=volume)
       
class itunes:
  
  def __init__(self):
    self.appscript = appscript.app('iTunes')   
    self.name = 'itunes'   
      
  def get_time(self):
    return self.appscript.get_time(2)
      
  def set_volume(self, volume):
    self.appscript.sound_volume.set(volume)

  def get_volume(self):
    return self.appscript.sound_volume.get()    
    
  def get_state(self):
    self.state = self.appscript.player_state.get().__repr__().split('.')[1]
    return self.state
  
  def playpause(self):
    self.appscript.playpause()

  def play(self):
    self.appscript.play()

  def pause(self):
    self.appscript.pause()    
              
  def stop(self):
    self.appscript.stop()
    
  def next(self):
    self.appscript.next()
  
class xfader:
  
  def __init__(self):
    
    pass
  
  def fade(self, options, fade_out):
    
    if fade_out == "itunes":
      fade_out_app = options.itunes
      fade_in_app = options.kjams
    else:
      fade_out_app = options.kjams
      fade_in_app = options.itunes
    
    while fade_in_app.get_volume() < options.max_volume:

      if fade_in_app.get_state() != "playing":
        fade_in_app.play()   
              
      print "Fading out %s, Fading in %s" % ( fade_out_app.name, fade_in_app.name)
      for item in [fade_out_app, fade_in_app]:
        print '%s is currently %s  Volume is %s.' % (item.name, item.get_state(), item.get_volume())
      fade_out_app.set_volume(fade_out_app.get_volume() - options.fade_step)
      fade_in_app.set_volume(fade_in_app.get_volume() + options.fade_step)
      time.sleep(options.fade_delay)

    #might want to stop kjams and just pause itunes 
    
    fade_in_app.set_volume(options.max_volume)
    fade_out_app.set_volume(options.min_volume)
     
    if fade_out_app.name == 'itunes':
      fade_out_app.pause()
    if fade_out_app.name == 'kjams':
      fade_out_app.stop()

    for item in [fade_out_app, fade_in_app]:
      print '%s is currently %s  Volume is %s.' % (item.name, item.get_state(), item.get_volume())

class application:
  
  def __init__(self):
    
    cfgdir = '%s/Library/Application Support/Faded' % os.environ.get("HOME")
    cfgfile = '%s/faded.ini' % cfgdir
    
    if not os.path.exists(cfgdir):
      os.makedirs(cfgdir)
    
    if not os.path.isfile(cfgfile):
      config = ConfigParser.RawConfigParser()
      config.add_section('config')
      config.set('config', 'seconds_left', '7')
      config.set('config', 'fade_step', '5')
      config.set('config', 'fade_delay', '.2')
      config.set('config', 'max_volume', '80')
      config.set('config', 'min_volume', '0')
      config.set('config', 'mix', 'auto')
    
      f = open(cfgfile, 'wb')
      config.write(f)
      f.close()
      
    config = ConfigParser.SafeConfigParser()
    config.read(cfgfile)
    self.seconds_left = int(config.get('config', 'seconds_left'))
    self.fade_step = int(config.get('config', 'fade_step'))
    self.fade_delay = float(config.get('config', 'fade_delay'))
    self.max_volume = int(config.get('config', 'max_volume'))  
    self.min_volume = int(config.get('config', 'min_volume') ) 
    self.mix = config.get('config', 'mix')    
    
    self.itunes = itunes()
    self.kjams  = kjams()
    self.xfader = xfader()    


  def main_loop(self):

    if self.mix == "auto":
      
      n = 0
      
      while n == 0:
        
        kjams_old_state = self.kjams.get_state()
        print "kJams old state is %s" % kjams_old_state      
        
        print time.sleep(1)
  
        for item in [self.itunes, self.kjams]:
          print '%s is currently %s  Volume is %s.' % (item.name, item.get_state(), item.get_volume())
        
        print '%s seconds left in kJams track' % ( str(self.kjams.get_time()) )
        print time.strftime("%a, %d %b %Y %H:%M:%S", time.localtime() )
        
        #if there is x seconds left in kjams, start fading out kjams and brining in itunes
        if self.kjams.get_time() <= self.seconds_left and self.kjams.get_state() == 'playing':
          print "There is %s or less seconds left in the current kjams track" % str(self.seconds_left)
          self.xfader.fade(self, "kjams")
        
        #if kjams has started playing, let's fade out itunes and fade in kjams  
        if kjams_old_state == 'stopped'  and self.kjams.get_state() != 'stopped':
           print "kJams has started playing... fading out iTunes"
           self.xfader.fade(self, "itunes")       
          
        if kjams_old_state == 'playing'  and self.kjams.get_state() == 'stopped':
          print "kJams has started playing... fading out kjams"
          self.xfader.fade(self, "kjams")
          
    else:
      
      if self.kjams.get_state() == 'playing':
        xfader.fade(self, "kjams")      
      else:
        self.xfader.fade(self, "itunes")         





if __name__ == "__main__" :
    
  #begin main application loop
  a = application()
  a.main_loop()
   
    

